@@ -1,8 +1,12 @@
 import import_string
+from pathlib import Path
+from uuid import uuid4
 
 from chamber.models import SmartModel
 from chamber.utils.datastructures import ChoicesNumEnum
 from django.core.exceptions import ValidationError
+from django.core.files.base import ContentFile
+from django.utils.functional import cached_property
 from django.db import models
 from django.utils.translation import ugettext, ugettext_lazy as _
 from django.template import Template, Context
@@ -11,7 +15,7 @@ from django.template.exceptions import TemplateSyntaxError, TemplateDoesNotExist
 from pymess.config import settings
 from pymess.utils.html import raise_error_if_contains_banned_tags
 
-from .common import BaseAbstractTemplate, BaseMessage, BaseRelatedObject
+from .common import BaseAbstractTemplate, BaseMessage, BaseRelatedObject, MessageQueryset
 
 
 __all__ = (
@@ -24,6 +28,22 @@ __all__ = (
 )
 
 
+def generate_content_filename(instance, filename):
+    return Path(settings.EMAIL_STORAGE_PATH) / 'contents' / 'content_{}.txt'.format(uuid4())
+
+
+def generate_attachment_filename(instance, filename):
+    return Path(settings.EMAIL_STORAGE_PATH) / 'attachments' / filename
+
+
+class EmailMessageQuerySet(MessageQueryset):
+
+    def create(self, content, **kwargs):
+        message = self.model(**kwargs)
+        message.content_file.save(None, ContentFile(content.encode()))
+        return message
+
+
 class EmailMessage(BaseMessage):
 
     STATE = ChoicesNumEnum(
@@ -56,6 +76,19 @@ class EmailMessage(BaseMessage):
         blank=True,
         editable=False,
     )
+    old_content = models.TextField(
+        verbose_name=_('content'),
+        null=True,
+        blank=True
+    )
+    content_file = models.FileField(
+        verbose_name=_('content file'),
+        null=True,
+        blank=True,
+        upload_to=generate_content_filename,
+    )
+
+    objects = EmailMessageQuerySet.as_manager()
 
     class Meta(BaseMessage.Meta):
         verbose_name = _('e-mail message')
@@ -75,6 +108,10 @@ class EmailMessage(BaseMessage):
     def failed(self):
         return self.state in {self.STATE.ERROR, self.STATE.ERROR_RETRY}
 
+    @cached_property
+    def content(self):
+        return self.content_file.read().decode() if self.content_file else self.old_content
+
 
 class EmailRelatedObject(BaseRelatedObject):
 
@@ -108,7 +145,8 @@ class Attachment(SmartModel):
     email_message = models.ForeignKey(EmailMessage, verbose_name=_('e-mail message'), on_delete=models.CASCADE,
                                       related_name='attachments')
     content_type = models.CharField(verbose_name=_('content type'), blank=False, null=False, max_length=100)
-    file = models.FileField(verbose_name=_('file'), null=False, blank=False, upload_to='pymess/emails')
+    file = models.FileField(verbose_name=_('file'), null=False, blank=False,
+                            upload_to=generate_attachment_filename)
 
     objects = AttachmentManager()
 
